<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO\Response\Notification;

abstract class Crud extends Simple {
	/** @var  string */
	protected $dataSourceName;
	/** @var Manager */
	protected $mongoManager;
	/** @var DeleteCascadeService */
	protected $deleteCascadeService;

	public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService) {
		$this->mongoManager = $mongoManager;
		$this->deleteCascadeService = $deleteCascadeService;
	}

	public function save(DTO\Request $request, DTO\Response $response) {
		try {
			$request = $this->buildDataSetIfRow($request);
			$dataSet = $request->getDataSet();
			$dataSet->setDataSourceName($this->dataSourceName);
			$persistedRows = $this->mongoManager->persist($dataSet);
			$nrPersistedRows = count($persistedRows);
			$response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
			$response->addNotification(new Notification("{$nrPersistedRows} row(s) persisted with success"));
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		$filterData = $request->getFilterCriteria();
		$filterData->setDataSourceName($this->dataSourceName);
		$dataSet = $this->mongoManager->findBy($filterData);
		$response->addDataSet($dataSet);
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		try {
			$request = $this->buildDataSetIfRow($request);
			$dataSet = $request->getDataSet();
			$dataSet->setDataSourceName($this->dataSourceName);
			$this->deleteCascadeService->deleteCascade($dataSet);
			$response->addNotification(new Notification("Row(s) deleted with success."));
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}	

	protected function buildDataSetIfRow(DTO\Request $request) {
		if ($request instanceof DTO\Request\Row) {
			$dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
			$request = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		}
		return $request;
	}

}