<?php

namespace Controller;


class Exception extends \Exception{

    const RESPONSE_CONNECTION_NOT_FOUND = "Connection not found: ";
    const RESPONSE_FAIL_AT_CREATE = "Could not create the connection: ";
    const BASE_RESPONSE_CONNECTION_FAIL = "Could not connect to the database.";
    const RESPONSE_CONNECTION_FAIL = "Could not connect to the database, the old columns were kept.";
    const BASE_RESPONSE_EXECUTE_QUERY_FAIL = "Failed to execute query. ";
    const RESPONSE_EXECUTE_QUERY_FAIL = "Could not sync your columns. Failed to execute query: ";
    const RESPONSE_SYNC_JOIN_DATASOURCE_COLUMNS = "Failed to sync columns. Could not find columns of original datasource ";
    const CODE_FAIL_CONNECTION = 404;
    const CODE_QUERY_EXECUTION_FAIL = 500;

    public static function buildErrorJSON(string $message, array $params = []){
        $error = array();
        $error["message"] = $message;
        if(!empty($params)){
            $error["params"] = $params;
        }
        return json_encode($error);
    }

    public static function connectionNotFound($connectionName = "") {
        return new static(self::RESPONSE_CONNECTION_NOT_FOUND.$connectionName, 1);
    }

    public static function createConnectionError($connectionName) {
        return new static(self::RESPONSE_FAIL_AT_CREATE.$connectionName, 1);
    }

    public static function baseConnectFail() {
        return new static(self::BASE_RESPONSE_CONNECTION_FAIL, self::CODE_FAIL_CONNECTION);
    }

    public static function connectFail() {
        return new static(self::RESPONSE_CONNECTION_FAIL, self::CODE_FAIL_CONNECTION);
    }

    public static function baseExecuteQueryError($query, \Exception $previousException) {
        return new static(self::BASE_RESPONSE_EXECUTE_QUERY_FAIL."Error: {$previousException->getMessage()}. The executed query was: \n {$query}", self::CODE_QUERY_EXECUTION_FAIL);
    }
    
    public static function executeQueryError($query, \Exception $previousException) {
        return new static(self::RESPONSE_EXECUTE_QUERY_FAIL."{$query}.\n Error: {$previousException->getMessage()}", self::CODE_QUERY_EXECUTION_FAIL);
    }
    
    public static function syncJoinDataSourceColumnsError($originalDataSourceName) {
        return new static(self::RESPONSE_SYNC_JOIN_DATASOURCE_COLUMNS."{$originalDataSourceName}");
    }
    
    public static function throwNewException($errorMessage){
        return new static($errorMessage);
    }
} 