<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Util\Environment;
use \Teknisa\Model\Geral\Entities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use \Service\Manager\Mongo\Manager;
use Service\Traits\TQuery;

class Operador{
    
    use TQuery;

    /** @var EntityManager $entityManager */
    protected  $entityManager;

    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;

    /** @var Environment $environment */
    protected  $environment;

    protected $projectId;

    public function __construct(EntityManager $entityManager, Environment $environment ,$projectId,
                                Manager $defaultOrganizationManager){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->defaultOrganizationManager = $defaultOrganizationManager;
        $this->projectId = $projectId;
    }

    public function currentClienteSql($orgId){
        $sql = <<<SQL
               SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,OMG.IDALLOWPRODUCTCALL,ORG.NRORG,
               ORG.NRORGPRINCIPAL
                FROM CLIENTE CLI
                JOIN ORGANIZACAO ORG ON(ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO)
           LEFT JOIN OM_PARAM_ORG OMG ON (OMG.NRORG = ORG.NRORG)
               WHERE ORG.NRORG = :ORG_ID
SQL;

        $param = array(
            'ORG_ID' => $orgId,
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $param);
    }

    public function findAvailableClienteSql($userId, $orgId, $filter = ''){
        $zmartIds = $this->findZmartOrg();
        if(!empty($zmartIds)){
            $queryZmart = ' AND NRORG IN ('. $zmartIds[0]['NRORG'].')';
        }else{
            $queryZmart = "";
        }

        $sql = <<<SQL
           SELECT NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,CDCLIENTE AS CLIENTE,NRORG,NRORGPRINCIPAL
           FROM(
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,ORG.NRORG,ORG.NRORGPRINCIPAL
                    FROM OPERCLIE OCL
                    JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = :CD_OPERADOR)
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDCLIEATIV = 'S')
                    JOIN ORGANIZACAO ORG ON ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                   UNION
                   SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,ORG.NRORG,ORG.NRORGPRINCIPAL
                    FROM OPERADOR OPE
                    JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDCLIEATIV = 'S')
                    JOIN ORGANIZACAO ORG ON ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                   WHERE OPE.CDOPERADOR = :CD_OPERADOR
                   UNION
                  SELECT CLI.NMFANTCLIE,CLI.NMRAZSOCCLIE,CLI.CDCLIENTE,CLI.NRINSJURCLIE,ORG.NRORG,ORG.NRORGPRINCIPAL
                    FROM CLIENTE CLI                    
                    JOIN ORGANIZACAO ORG ON ORG.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                    JOIN OPERADOR OPE ON (OPE.CDOPERADOR = :CD_OPERADOR AND OPE.IDOPERMASTERZMART = 'S')
                   WHERE CLI.IDCLIEATIV = 'S'
                )
           WHERE 1 = 1
           $queryZmart
           $filter
           GROUP BY NMFANTCLIE,NMRAZSOCCLIE,CDCLIENTE,NRINSJURCLIE,NRORG,NRORGPRINCIPAL
           ORDER BY NMFANTCLIE ASC
SQL;

        $param = array(
            'ORG_ID' => $orgId,
            'CD_OPERADOR' => $userId,
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    public  function  findZmartOrg(){
        $sql = <<<SQL
                 SELECT rtrim(xmlagg(xmlelement(e,NRORG,',').extract('//text()') order by NRORG).GetClobVal(),',') AS NRORG
                 FROM (
                    SELECT ORG.NRORG,OPR.NREXTERNAL_PRODUCT_ID 
                         FROM OM_PRODUTO OPR
                         JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OPR.ID
                         JOIN ORGANIZACAO ORG ON (ORG.NRORG = OPO.NRORG_ID)
                        WHERE OPR.NREXTERNAL_PRODUCT_ID = :ZMART_ID
                          AND ORG.IDATIVO != 'N'
                    UNION
                     SELECT ORG.NRORGPRINCIPAL AS NRORG, OPR.NREXTERNAL_PRODUCT_ID
                         FROM OM_PRODUTO OPR
                         JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OPR.ID
                         JOIN ORGANIZACAO ORG ON (ORG.NRORG = OPO.NRORG_ID)
                        WHERE OPR.NREXTERNAL_PRODUCT_ID = :ZMART_ID
                          AND ORG.IDATIVO != 'N'
                          AND ORG.NRORGPRINCIPAL IS NOT NULL
                  )
                 GROUP BY NREXTERNAL_PRODUCT_ID
SQL;

        $param = array(
            'ZMART_ID' => $this->projectId
        );
        return \Util\InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true)->getConnection()->fetchAll($sql,$param);

    }

    public  function  findZmartOrgToMongo(){
        $sql = <<<SQL
           SELECT OPO.NRORG_ID AS NRORG
             FROM OM_PRODUTO OPR
             JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OPR.ID
             JOIN ORGANIZACAO ORG ON (ORG.NRORG = OPO.NRORG_ID)
            WHERE OPR.NREXTERNAL_PRODUCT_ID = :ZMART_ID
              AND ORG.IDATIVO != 'N'
            GROUP BY OPO.NRORG_ID
SQL;

        $param = array(
            'ZMART_ID' => $this->projectId
        );

        return \Util\InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true)->getConnection()->fetchAll($sql,$param);

    }

    public function  clientVerification($clientes){
        $values = array();
        foreach ($clientes as &$cliente) {
            if(strlen($cliente['NRINSJURCLIE']) == 14){
                $cliente['NRINSJURCLIE'] = preg_replace('/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/', "$1.$2.$3/$4-$5",$cliente['NRINSJURCLIE']);
            }

            array_push($values,strval($cliente['NRORG']));
            array_push($values,intval($cliente['NRORG']));
        }

        $filterCriteria = new FilterCriteria("connection", array(
                array(
                    "columnName" => "organizationId",
                    "value" => $values,
                    "operator" => "IN")
            )
        );

        $rows = $this->defaultOrganizationManager->findBy($filterCriteria)->getRows();

        $clientesWithConnection = array();
        foreach ($clientes as &$cliente) {
            foreach ($rows as $row) {
                if($cliente['NRORG'] == $row['organizationId']){
                    array_push($clientesWithConnection,$cliente);
                }
            };
        };

        return $clientesWithConnection;
    }


    public function findAvailableCliente($condition){
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        $clientes = $this->findAvailableClienteSql($userId, 0);
        $clientesWithConnection['data'] = $this->clientVerification($clientes['data'] );
        return $clientesWithConnection;
    }

    public function findAllAvailableCliente($condition){
        $filter = Utilities::createQueryFiltered($condition);
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        $clientes = $this->operadorRepository->findAvailableCliente($userId, 0, $filter);
        $clientesWithConnection = $this->clientVerification($clientes);
        return $clientesWithConnection;
    }

    public function currentCliente(){
        $orgId = $this->environment->getNrOrgClient();
        if(!$orgId){
            $orgId = Utilities::getEnvironment()->getCurrentOrganizationId();
        }
        return $this->currentClienteSql($orgId);
    }

    public function saveCurrentCliente($organizationId){
        Utilities::getEnvironment()->setOrganizationId($organizationId);
        Utilities::getEnvironment()->setNrOrg($organizationId);
    }

}
